import 'dart:async';
import 'dart:io';

import 'package:async_task/async_task.dart';
import 'package:dio/dio.dart';

import 'api_interceptors.dart';

const _defaultConnectionTimeout = 30 * 1000;
const _defaultReceiveTimeout = 30 * 1000;

typedef KntExecutor = Future<String> Function();

final pohDio = Dio(BaseOptions(
  baseUrl: 'https://api.poh.com.vn',
  connectTimeout: _defaultConnectionTimeout,
  receiveTimeout: _defaultReceiveTimeout,
));

Future<String> pohExecutor() async {
  await Future.delayed(const Duration(seconds: 20));
  final response = await pohDio.post(
    '/api/v1/auth/login',
    data: {
      'email': 'dangnh.poh@gmail.com',
      'password': 'BeoCon21',
    },
  );
  return response.data.toString();
}

class ConcurrentTask extends AsyncTask<void, String> {
  @override
  AsyncTask<void, String> instantiate(void parameters,
      [Map<String, SharedData>? sharedData]) {
    return ConcurrentTask();
  }

  @override
  void parameters() {}

  @override
  FutureOr<String> run() async {
    return await pohExecutor.call();
  }
}

List<AsyncTask> _taskTypeRegister() => [ConcurrentTask()];

class PohRepo {
  Dio? _dio;

  Future<void> init() async {
    _dio = await _createDio('https://api.poh.com.vn');
  }

  Future<String> login({bool needConcurrent = false}) async {
    Future<String> executor() async {
      final dio = _dio;
      await Future.delayed(const Duration(seconds: 20));
      if (dio != null) {
        final response = await dio.post(
          '/api/v1/auth/login',
          data: {
            'email': 'dangnh.poh@gmail.com',
            'password': 'BeoCon21',
          },
        );
        return response.data.toString();
      }
      return '';
    }

    if (!needConcurrent) {
      return await executor();
    }

    final task = ConcurrentTask();
    final tasks = [task];

    // Instantiate the task executor:
    final asyncExecutor = AsyncExecutor(
      sequential: false,
      parallelism: 1,
      taskTypeRegister: _taskTypeRegister,
    );

    // Enable logging output:
    asyncExecutor.logger.enabled = true;

    // Execute all tasks:
    var executions = asyncExecutor.executeAll(tasks);

    // Wait tasks executions:
    await Future.wait(executions);

    return tasks.first.result ?? 'Error Async Task';
  }

  Future<Dio> _createDio(
    String baseUrl, {
    int? connectTimeout,
    int? receiveTimeout,
    bool needAuthorized = false,
    Map<String, dynamic> customHeaders = const {},
  }) async {
    Dio dio = Dio(BaseOptions(
      baseUrl: baseUrl,
      connectTimeout: connectTimeout ?? _defaultConnectionTimeout,
      receiveTimeout: receiveTimeout ?? _defaultReceiveTimeout,
      headers: customHeaders,
    ));
    if (needAuthorized) {
      const token = '';
      dio.interceptors
          .add(InterceptorsWrapper(onRequest: (options, handlers) async {
        dio.lock();
        options.headers[HttpHeaders.authorizationHeader] = 'Bearer $token';
        options.headers[HttpHeaders.contentTypeHeader] =
            'application/json; charset=UTF-8';
        dio.unlock();
      }));
    }
    dio.interceptors.add(PrettyDioLogger(
      request: true,
      requestHeader: true,
    ));
    return dio;
  }
}
