import 'dart:ui';

import '../generated/colors.gen.dart';
import 'widget/image/knt_local_image.dart';

abstract class KntUi {
  static _KntUiConfig _shared = _KntUiConfig(
    primaryColor: ColorName.jungleGreen,
    secondaryColor: ColorName.doveGray2,
  );

  static _KntUiConfig get shared => _shared;

  static config({
    required Color primaryColor,
    required Color secondaryColor,
    Color? shimmerColor,
    AbsolutePathConverter? filePathConverter,
    String? assetsErrorPath,
  }) {
    _shared = _KntUiConfig(
      primaryColor: primaryColor,
      secondaryColor: secondaryColor,
      shimmerColor: shimmerColor,
      filePathConverter: filePathConverter,
      assetsErrorPath: assetsErrorPath,
    );
  }
}

class _KntUiConfig {
  /// Usually in used for app bar/tool bar/progress/...
  final Color primaryColor;

  /// Usually in used for sub title text/background/...
  final Color secondaryColor;

  /// Use for shimmer's effect
  final Color? shimmerColor;

  /// Use for convert a relative photo path (usually iOS) into an absolute path
  final AbsolutePathConverter? filePathConverter;

  /// Use for showing an widget which presented any issue/error
  final String? assetsErrorPath;

  const _KntUiConfig({
    required this.primaryColor,
    required this.secondaryColor,
    this.shimmerColor,
    this.filePathConverter,
    this.assetsErrorPath,
  });
}
