import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';

import '../../model/storage_type.dart';
import 'knt_cached_image.dart';
import 'knt_local_image.dart';

class KntImage extends StatelessWidget {
  final String uri;
  final double? width;
  final double? height;
  final Widget? child;
  final Widget? placeholder;
  final BorderRadius? borderRadius;
  final BoxFit? fit;
  final StorageType imageType;
  final AbsolutePathConverter? converter;
  final Map<String, String>? httpHeaders;

  const KntImage({
    Key? key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.borderRadius,
    this.fit,
    this.imageType = StorageType.assets,
    this.converter,
    this.httpHeaders,
  }) : super(key: key);

  const KntImage.network({
    Key? key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.borderRadius,
    this.fit,
    this.httpHeaders,
  })  : this.imageType = StorageType.network,
        this.converter = null,
        super(key: key);

  const KntImage.assets({
    Key? key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.borderRadius,
    this.fit,
  })  : this.imageType = StorageType.assets,
        this.converter = null,
        this.httpHeaders = null,
        super(key: key);

  const KntImage.file({
    Key? key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.borderRadius,
    this.fit,
    this.converter,
  })  : this.imageType = StorageType.file,
        this.httpHeaders = null,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    switch (imageType) {
      case StorageType.network:
        return KntCachedImage(
          url: uri,
          width: width,
          height: height,
          child: child,
          placeholder: placeholder,
          borderRadius: borderRadius,
          fit: fit,
          httpHeaders: httpHeaders,
        );
      case StorageType.assets:
        final content = uri.endsWith('.svg')
            ? SvgPicture.asset(
                uri,
                width: width,
                height: height,
                placeholderBuilder: (_) => this.placeholder ?? const SizedBox(),
                fit: fit ?? BoxFit.cover,
              )
            : Image.asset(
                uri,
                width: width,
                height: height,
                errorBuilder: (_, __, ___) =>
                    this.placeholder ?? const SizedBox(),
                fit: fit ?? BoxFit.cover,
              );
        return borderRadius != null
            ? ClipRRect(
                borderRadius: borderRadius,
                child: content,
              )
            : content;
      case StorageType.file:
        return KntLocalImage(
          relativePath: uri,
          converter: converter,
          width: width,
          height: height,
          child: child,
          placeholder: placeholder,
          borderRadius: borderRadius,
          fit: fit,
        );
    }
  }
}
