import 'dart:io';

import 'package:flutter/material.dart';

import '../../config.dart';
import '../knt_shimmer.dart';
import 'knt_default_image.dart';

typedef AbsolutePathConverter = Future<String> Function(String);

class KntLocalImage extends StatefulWidget {
  final String? relativePath;
  final AbsolutePathConverter? converter;
  final double? width;
  final double? height;
  final BorderRadius? borderRadius;
  final Border? border;
  final List<BoxShadow>? boxShadows;
  final EdgeInsets? padding;
  final Widget? child;
  final BoxFit? fit;
  final Widget? placeholder;
  final Widget? errorImage;

  const KntLocalImage({
    Key? key,
    this.relativePath,
    this.converter,
    this.width,
    this.height,
    this.borderRadius,
    this.border,
    this.boxShadows,
    this.padding,
    this.child,
    this.fit,
    this.placeholder,
    this.errorImage,
  }) : super(key: key);

  @override
  _KntLocalImageState createState() => _KntLocalImageState();
}

class _KntLocalImageState extends State<KntLocalImage> {
  late bool _loading;
  File? _imageFile;

  @override
  void initState() {
    super.initState();
    _loading = true;
    _loadData();
  }

  @override
  void didUpdateWidget(covariant KntLocalImage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.relativePath != oldWidget.relativePath) {
      setState(() {
        _loading = true;
      });
      _loadData();
    }
  }

  void _loadData() async {
    var fullPath = widget.relativePath ?? '';
    if (widget.converter == null) {
      if (KntUi.shared.filePathConverter != null) {
        fullPath = await KntUi.shared.filePathConverter!.call(fullPath);
      }
    } else {
      fullPath = await widget.converter!.call(fullPath);
    }
    setState(() {
      _imageFile = File(fullPath);
      _loading = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    final defaultImage = KntDefaultImage(
      width: widget.width,
      height: widget.height,
      borderRadius: widget.borderRadius,
      border: widget.border,
      boxShadows: widget.boxShadows,
    );
    if (_loading) {
      return widget.placeholder ??
          KntShimmer(
            width: widget.width,
            height: widget.height,
            borderRadius: widget.borderRadius,
            border: widget.border,
            boxShadows: widget.boxShadows,
          );
    }
    if (_imageFile == null || !_imageFile!.existsSync()) {
      return widget.placeholder ?? defaultImage;
    }
    return Container(
      width: widget.width,
      height: widget.height,
      padding: widget.padding,
      decoration: BoxDecoration(
        image: DecorationImage(
          image: FileImage(_imageFile!),
          fit: widget.fit ?? BoxFit.cover,
        ),
        borderRadius: widget.borderRadius,
        border: widget.border,
        boxShadow: widget.boxShadows,
      ),
      child: widget.child,
    );
  }
}
