import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../config.dart';
import 'knt_ripple.dart';

class KntButton extends StatelessWidget {
  final String title;
  final bool needFullWidth;
  final Widget? leadingIcon;
  final Widget? trailingIcon;
  final Color? textColor;
  final Color? backgroundColor;
  final BorderRadius? borderRadius;
  final VoidCallback? onPressed;

  const KntButton({
    Key? key,
    required this.title,
    this.needFullWidth = true,
    this.leadingIcon,
    this.trailingIcon,
    this.textColor,
    this.backgroundColor,
    this.borderRadius,
    this.onPressed,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ElevatedButton(
      onPressed: onPressed,
      style: ElevatedButton.styleFrom(
        primary: backgroundColor ?? KntUi.shared.primaryColor,
        shape: RoundedRectangleBorder(
          borderRadius: borderRadius ?? BorderRadius.circular(3.w),
        ),
      ),
      child: Container(
        width: needFullWidth ? 1.sw : null,
        height: 48.w,
        padding: EdgeInsets.symmetric(vertical: 16.w),
        alignment: Alignment.center,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            if (leadingIcon != null) leadingIcon!,
            if (leadingIcon != null) SizedBox(width: 12.w),
            Text(
              title,
              maxLines: 1,
              style: TextStyle(
                color: textColor ?? Colors.white,
                fontSize: 15.sp,
                fontWeight: FontWeight.w700,
                height: 15.sp / 15.sp,
              ),
            ),
            if (trailingIcon != null) SizedBox(width: 12.w),
            if (trailingIcon != null) trailingIcon!,
          ],
        ),
      ),
    );
  }
}

class KntCircleIconButton extends StatelessWidget {
  final Widget icon;
  final VoidCallback? onTap;

  const KntCircleIconButton({
    Key? key,
    required this.icon,
    this.onTap,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return RawMaterialButton(
      onPressed: onTap,
      constraints: BoxConstraints(
        minWidth: 36.w,
        maxWidth: 36.w,
        minHeight: 36.w,
        maxHeight: 36.w,
      ),
      child: icon,
      shape: CircleBorder(),
    );
  }
}

class KntSimpleButton extends StatelessWidget {
  final double? width;
  final Widget? title;
  final Widget? subTitle;
  final Color? backgroundColor;
  final EdgeInsets? margin;
  final VoidCallback? onTap;

  const KntSimpleButton({
    Key? key,
    this.width,
    this.title,
    this.subTitle,
    this.backgroundColor,
    this.margin,
    this.onTap,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return KntRipple(
      onTap: onTap,
      child: Container(
        width: width ?? 1.sw,
        margin: margin ?? const EdgeInsets.all(0.0),
        padding: EdgeInsets.all(12.w),
        decoration: BoxDecoration(
          color: onTap == null
              ? Colors.grey
              : (backgroundColor ?? KntUi.shared.primaryColor),
          borderRadius: BorderRadius.circular(8.w),
        ),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            title ?? const SizedBox(),
            subTitle ?? const SizedBox(),
          ],
        ),
      ),
    );
  }
}
