import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

class KntDraggablePanel extends StatelessWidget {
  final double? headerHeight;
  final double? paddingTopHeader;
  final double? paddingWidth;
  final Widget? child;
  final List<Widget>? slivers;
  final Color? backgroundColor;
  final BorderRadius? borderRadius;
  final List<BoxShadow>? boxShadowList;

  const KntDraggablePanel({
    Key? key,
    this.headerHeight,
    this.paddingTopHeader,
    this.paddingWidth,
    this.child,
    this.slivers,
    this.backgroundColor,
    this.borderRadius,
    this.boxShadowList,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final headerSize =
        headerHeight ?? (MediaQuery.of(context).padding.top + 64.w);
    final minChildSize = 1 - ((paddingTopHeader ?? 0.0) + headerSize) / 1.sh;
    final maxChildSize = 1 - headerSize / 1.sh;
    return DraggableScrollableSheet(
      initialChildSize: minChildSize,
      minChildSize: minChildSize,
      maxChildSize: maxChildSize,
      builder: (context, scrollController) {
        return Container(
          width: 1.sw,
          padding: EdgeInsets.symmetric(horizontal: paddingWidth ?? 24.w),
          decoration: BoxDecoration(
            color: backgroundColor,
            borderRadius: borderRadius,
            boxShadow: boxShadowList,
          ),
          child: slivers != null
              ? CustomScrollView(
                  controller: scrollController,
                  physics: const BouncingScrollPhysics(),
                  slivers: slivers!,
                )
              : SingleChildScrollView(
                  controller: scrollController,
                  physics: const BouncingScrollPhysics(),
                  child: child,
                ),
        );
      },
    );
  }
}
