import 'package:flutter/material.dart';

const _durationExpandingInMillis = Duration(milliseconds: 400);

class KntExpandableWidget extends StatefulWidget {
  final Widget child;
  final bool expandedAtFirst;
  final Duration? duration;

  const KntExpandableWidget({
    Key? key,
    required this.child,
    this.expandedAtFirst = false,
    this.duration,
  });

  @override
  _KntExpandableWidgetState createState() => _KntExpandableWidgetState();
}

class _KntExpandableWidgetState extends State<KntExpandableWidget>
    with SingleTickerProviderStateMixin {
  late AnimationController _expandController;
  late Animation<double> _animation;

  @override
  void initState() {
    super.initState();
    _prepareAnimations();
    _runExpandCheck();
  }

  @override
  void didUpdateWidget(KntExpandableWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    _runExpandCheck();
  }

  @override
  void dispose() {
    _expandController.dispose();
    super.dispose();
  }

  void _prepareAnimations() {
    _expandController = AnimationController(
      vsync: this,
      duration: widget.duration ?? _durationExpandingInMillis,
    );
    _animation = CurvedAnimation(
      parent: _expandController,
      curve: Curves.fastOutSlowIn,
    );
  }

  void _runExpandCheck() {
    if (widget.expandedAtFirst) {
      _expandController.forward();
    } else {
      _expandController.reverse();
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizeTransition(
      axisAlignment: 1.0,
      sizeFactor: _animation,
      child: widget.child,
    );
  }
}
