import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../config.dart';
import 'knt_edit_text.dart';

class KntSearchBar extends StatefulWidget {
  final String? defaultSearchText;
  final String? searchHint;
  final Widget? iconSearch;
  final Function(String)? onTextChanged;

  const KntSearchBar({
    Key? key,
    this.defaultSearchText,
    this.searchHint,
    this.iconSearch,
    this.onTextChanged,
  }) : super(key: key);

  @override
  _KntSearchBarState createState() => _KntSearchBarState();
}

class _KntSearchBarState extends State<KntSearchBar> {
  Timer? _debounceSearching;
  String? _searchText;

  @override
  void initState() {
    super.initState();
    _initData();
  }

  @override
  void didUpdateWidget(covariant KntSearchBar oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_searchText != widget.defaultSearchText) {
      setState(_initData);
    }
  }

  @override
  void dispose() {
    _debounceSearching?.cancel();
    super.dispose();
  }

  void _initData() {
    _searchText = widget.defaultSearchText;
  }

  void _onTextChanged(String newText) {
    // Debounce to prevent too much query at the same time
    if (_debounceSearching?.isActive == true) {
      _debounceSearching?.cancel();
    }
    _debounceSearching = Timer(const Duration(milliseconds: 500), () {
      _onUpdateText(newText);
    });
  }

  void _onClearSearch() {
    if (_searchText == '') {
      return;
    }
    _onUpdateText('');
  }

  void _onUpdateText(String text) {
    setState(() {
      _searchText = text;
    });
    widget.onTextChanged?.call(text);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 48.w,
      padding: EdgeInsets.symmetric(horizontal: 16.w),
      decoration: BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.circular(12.w),
        boxShadow: [
          BoxShadow(
            color: Colors.black12,
            offset: Offset(0.0, 2.w),
            blurRadius: 4.0,
            spreadRadius: 2.0,
          ),
        ],
      ),
      child: Row(
        children: [
          widget.iconSearch ??
              Icon(
                Icons.search,
                size: 24.w,
                color: KntUi.shared.secondaryColor,
              ),
          SizedBox(width: 12.w),
          Expanded(
            child: KntSimpleEditText(
              defaultValue: _searchText,
              hintText: widget.searchHint,
              hintTextColor: KntUi.shared.secondaryColor,
              fontSize: 15.sp,
              onTextChanged: _onTextChanged,
              maxLine: 1,
            ),
          ),
          SizedBox(width: 8.w),
          InkWell(
            onTap: _onClearSearch,
            child: Container(
              width: 48.w,
              alignment: Alignment.centerRight,
              child: Icon(
                Icons.close,
                size: 20.w,
                color: KntUi.shared.secondaryColor,
              ),
            ),
          ),
        ],
      ),
    );
  }
}
