import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../config.dart';

class KntToolBar extends StatelessWidget {
  final String titlePage;
  final Widget? leadingIcon;
  final List<Widget>? trailingActions;
  final Color? backgroundColor;
  final Color? titleColor;
  final bool includeStatusBar;

  const KntToolBar({
    Key? key,
    required this.titlePage,
    this.leadingIcon,
    this.trailingActions,
    this.backgroundColor,
    this.titleColor,
    this.includeStatusBar = true,
  }) : super(key: key);

  void _onDismiss(BuildContext context) {
    Navigator.of(context).pop();
  }

  @override
  Widget build(BuildContext context) {
    final topSpacing =
        includeStatusBar ? MediaQuery.of(context).padding.top : 0.0;
    return Container(
      width: 1.sw,
      height: 56.w + topSpacing,
      padding: EdgeInsets.only(
        top: topSpacing,
        left: 8.w,
        right: 8.w,
      ),
      decoration: BoxDecoration(
        color: backgroundColor ?? KntUi.shared.primaryColor,
        boxShadow: [
          BoxShadow(
            color: Colors.black12.withAlpha(20),
            offset: Offset(-1.w, 1.w),
            blurRadius: 1.w,
            spreadRadius: 1.w,
          ),
        ],
      ),
      child: Stack(
        children: [
          Container(
            height: 56.w,
            child: Row(
              children: [
                leadingIcon != null
                    ? IconButton(
                        icon: leadingIcon!,
                        onPressed: () => _onDismiss(context),
                      )
                    : const SizedBox(),
                const Spacer(),
                Row(
                  mainAxisAlignment: MainAxisAlignment.end,
                  children: trailingActions ?? [],
                ),
              ],
            ),
          ),
          Align(
            child: Text(
              titlePage,
              maxLines: 1,
              overflow: TextOverflow.ellipsis,
              style: TextStyle(
                color: titleColor ?? Colors.white,
                fontSize: 16.sp,
                height: 18.sp / 16.sp,
                fontWeight: FontWeight.w600,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class KntDefaultToolBar extends StatelessWidget {
  final String titlePage;
  final List<Widget>? trailingActions;
  final bool includeStatusBar;

  const KntDefaultToolBar({
    Key? key,
    required this.titlePage,
    this.trailingActions,
    this.includeStatusBar = true,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return KntToolBar(
      titlePage: titlePage,
      leadingIcon: Icon(
        Icons.arrow_back_rounded,
        color: Colors.white,
        size: 20.w,
      ),
      trailingActions: trailingActions,
    );
  }
}
